Authenticatable 
==============
[![RuboCop Github Action](https://github.com/kiqr/authenticatable/actions/workflows/rubocop.yml/badge.svg)](https://github.com/kiqr/authenticatable/actions/workflows/rubocop.yml)
[![RSpec](https://github.com/kiqr/authenticatable/actions/workflows/rspec.yml/badge.svg)](https://github.com/kiqr/authenticatable/actions/workflows/rspec.yml)
[![codecov](https://codecov.io/gh/kiqr/authenticatable/branch/main/graph/badge.svg?token=UZMGXQKJRL)](https://codecov.io/gh/kiqr/authenticatable)
[![MIT License](https://img.shields.io/badge/License-MIT-blue.svg)](LICENSE.md)

An authentication solution for Rails that works on top of Rails built-in [has_secure_password](https://api.rubyonrails.org/classes/ActiveModel/SecurePassword/ClassMethods.html). Authenticatable ships with controllers, helpers, views and model generators. The purpose of Authenticatable is to enable you to kickstart a new Rails project without having to spend time on writing repetitive code that shouldn't have to be repeated in each project.

Authenticatable also ships with some **extra security features** that very often are forgotten by developers and the authors of the many available [has_secure_password](https://api.rubyonrails.org/classes/ActiveModel/SecurePassword/ClassMethods.html) tutorials:
- Protection against [sessions fixation attacks](https://guides.rubyonrails.org/security.html#session-fixation) by clearing the session_id on authentication.
- Protection against [cross-site request forgery (CSRF)](https://guides.rubyonrails.org/security.html#cross-site-request-forgery-csrf) by cleaning up the CSRF Token on authentication.
- Protection against [timing/enumeration attacks](https://www.kaspersky.com/blog/username-enumeration-attack/34618/) by hashing the password even if a user record isn't found.

#### What's included?
- Controllers and views for authentication, registration & reset password with support for both [turbo](https://github.com/hotwired/turbo-rails) and [turbolinks](https://github.com/turbolinks/turbolinks).
- Support for multiple user models (like users/accounts/admins or whatever you want).
- Custom identifier column (sign in with for example username or phonenumber instead of email).
- Customizable: Easily override default behaviors with your own.

#### Supported Rails/Ruby versions
Tests run against Rails versions `~> 5.2.0`, `~> 6.0.0`, `~> 6.1.0` and `~> 7.0.0.alpha2` under Ruby versions `2.6`, `2.7` and `3.0`

Installation
------------

Add the following line to Gemfile:

```ruby
gem "authenticatable", "~> 1.0"
```

and run `bundle install` from your terminal to install it.

Getting started
---------------

The first step is to create an authenticatable model. To generate an authenticatable model run the following command:

```console
$ rails g authenticatable NAME
```

This will generate a model with the given ```NAME``` (if one does not exist) with configuration for authenticatable, a migration file and routes. The output should be something similar to:

```console
foo@bar:~$ rails g authenticatable user
Running via Spring preloader in process 99920
      invoke  active_record
      create    db/migrate/20210909215956_authenticatable_create_users.rb
      create    app/models/user.rb
      invoke    test_unit
      create      test/models/user_test.rb
      create      test/fixtures/users.yml
      insert    app/models/user.rb
       route    authenticatable :users
```

### Securing your application

To set up a controller with authentication, just add the before_action `authenticate_{scope}!`. To require a `User` to be signed in:

```ruby
before_action :authenticate_user! # Block unauthenticated requests to the current controller
```

To restrict your whole application to signed-in users, you can add the before_action above to your ```ApplicationController```.

#### Handling Authenticatable::UnauthenticatedError

The `Authenticatable::UnauthenticatedError` exception is raised when calling `:authenticate_user!` in the controller and no valid user is signed in. You can catch the exception and modify its behaviour in the ApplicationController. The behavior may vary depending on the request format and user scope. For example here we set a flash error message and redirect to the sign in page for HTML requests and return 403 Forbidden for JSON requests.

```ruby
class ApplicationController < ActionController::Base
  before_action :authenticate_user!
  rescue_from Authenticatable::UnauthenticatedError do
    respond_to do |format|
      format.json { head :forbidden }
      format.html { redirect_to new_user_session_path, alert: "Please sign in to continue." }
    end
  end
end
```

Customizing
-----------

### Views
You probably want to (and should) modify the default Authenticatable views. To copy the views to your application run the following generator:

```console
$ rails g authenticatable:views
```

### Changing the model identifier
By default, a user is identified by `email` and password when logging in. But maybe you want your users to login using a `username` or something else instead of email. Luckely, that's easy done with Authenticatable:

```ruby
class User < ApplicationRecord
  authenticatable
  
  # Identifying column to use when looking up an authenticatable record in the database.
  # Can be for example email or a username. Default is email.
  identify_by :username
  
  # You need to create your own validator for your identifier column.
  validates_presence_of :username
end
```

Just make sure that the `username` column exists with a **unique index** in your database:

```shell
$ rails g migration add_username_to_users username:string:uniq
```

Contributing
------------
If you are interested in reporting/fixing issues and contributing directly to the code base, please see [CONTRIBUTING.md](CONTRIBUTING.md) for more information on what we're looking for and how to get started.

Versioning
----------
This library aims to adhere to [Semantic Versioning 2.0.0](http://semver.org/). Violations
of this scheme should be reported as bugs. Specifically, if a minor or patch
version is released that breaks backward compatibility, that version should be
immediately yanked and/or a new version should be immediately released that
restores compatibility. Breaking changes to the public API will only be
introduced with new major versions. As a result of this policy, you can (and
should) specify a dependency on this gem using the [Pessimistic Version
Constraint](http://guides.rubygems.org/patterns/#pessimistic-version-constraint) with two digits of precision. For example:

```ruby
gem "authenticatable", "~> 1.0"
```

License
-------
The gem is available as open source under the terms of the [MIT License](https://opensource.org/licenses/MIT).
